@@ -1,19 +1,25 @@
 #!/usr/bin/perl
-#$Id: stat.pl 591 2010-01-29 17:44:19Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/examples/stat/stat.pl $
+#$Id: stat.pl 686 2010-12-16 00:02:50Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/examples/stat/stat.pl $
 package statpl;
 use strict;
 no warnings qw(uninitialized);
-our ( %config, $param, $db, );
-use statlib;
+our ( %config, %static, $param, $db, );
 use Data::Dumper;
 $Data::Dumper::Sortkeys = $Data::Dumper::Useqq = $Data::Dumper::Indent = 1;
-use psmisc;
-our $root_path;
-use lib $root_path. '../../lib';
-use lib $root_path. './';
+use lib::abs qw(../../lib ./);
+use Net::DirectConnect::pslib::psmisc;    # qw(:config :log printlog);
+psmisc->import qw(:log);
+#*statpl::config = *main::config;
+#our $root_path;
+#use lib $root_path. '../../lib';
+#use lib $root_path. './';
 use Net::DirectConnect;
+#psmisc::configure();
+use statlib;
+#warn Dumper \%config, \%psmisc::config, \%statlib::config, \%statpl::config, \%main::config, \%pssql::config,;
+#warn Dumper \%INC, \@INC;
 $config{'queue_recalc_every'} ||= 60;
-$static{'no_sig_log'} = 1;    #test
+$static{'no_sig_log'} = 1;                #test
 print(
   "usage:
  $0 [--configParam=configValue] [adc|dchub://]host[:port] [more params and hubs]\n
@@ -23,7 +29,7 @@ print(
   exit
   if !$ARGV[0];
 my $n = -1;
-
+my ( $tq, $rq, $vq ) = $db->quotes();
 for my $arg (@ARGV) {
   ++$n;
   #print "ar[$arg]";
@@ -55,7 +61,7 @@ for my $arg (@ARGV) {
         )
       {
         next if $tim eq 'r' and ( !$config{'queries'}{$query}{'periods'} or $time eq 'h' );
-        printlog 'info', 'calculating ', $time, $query;
+        psmisc::printlog 'info', 'calculating ', $time, $query;
         local $config{'queries'}{$query}{'WHERE'}[5] =
           $config{'queries'}{$query}{'FROM'} . ".time >= " . int( time - $config{'periods'}{$time} )
           if $time;
@@ -63,16 +69,29 @@ for my $arg (@ARGV) {
         my $n = 0;
         for my $row (@$res) {
           ++$n;
-          my $dmp = Data::Dumper->new( [$row] )->Indent(0)->Terse(1)->Purity(1)->Dump();
-          $db->insert_hash( 'slow', { 'name' => $query, 'n' => $n, 'result' => $dmp, 'period' => $time, 'time' => int(time) } )
+          my $dmp = Data::Dumper->new( [$row] )->Indent(0)->Pair('=>')->Terse(1)->Purity(1)->Dump();
+          #warn "SLOWi:[$config{'use_slow'}][$dmp]";
+          $db->insert_hash( 'slow', { 'name' => $query, 'n' => $n, 'result' => $dmp, 'period' => $time, 'time' => $nowtime } )
             if $config{'use_slow'};
           #if ( $time eq 'd' ) {
           my $table = $query . '_' . $time;
           $table =~ s/\s/_/g;
-          $db->insert_hash( $table, { 'n' => $n, 'date' => psmisc::human('date'), %$row, 'time' => $nowtime } );
+          #print Dumper $row;
+          #warn $date;
+          $db->insert_hash(
+            $table, {
+              'n' => $n,
+              , %$row,
+              'time' => $nowtime,
+              'date' => psmisc::human( 'date', $nowtime )
+                . ( $tim ne 'h' ? '' : '-' . sprintf '%02d', ( localtime $nowtime )[2] ),
+            }
+          );
           #}
         }
-        $db->do( "DELETE FROM slow WHERE name=" . $db->quote($query) . " AND period=" . $db->quote($time) . " AND n>$n " )
+        #exit;
+        $db->do(
+          "DELETE FROM ${tq}slow${tq} WHERE name=" . $db->quote($query) . " AND period=" . $db->quote($time) . " AND n>$n " )
           if $config{'use_slow'};
         #$db->flush_insert('slow');
         $db->flush_insert();
@@ -85,10 +104,13 @@ for my $arg (@ARGV) {
     for my $table ( sort keys %{ $config{'sql'}{'table'} } ) {
       #print "$table  \n";
       my ($col) = grep { $config{'sql'}{'table'}{$table}{$_}{'purge'} } keys %{ $config{'sql'}{'table'}{$table} };
+      #printlog('err', "no col in [$table]", Dumper $config{'sql'}{'table'}{$table}),
+      next unless $col;
       my $purge = $config{'sql'}{'table'}{$table}{$col}{'purge'};
       #print "t $table c$col p$purge \n";
       $purge = $config{'purge'} if $purge and $purge <= 1;
-      printlog 'info', "purge $table $col $purge =", $db->do( "DELETE FROM $table WHERE $col < " . int( time - $purge ) );
+      psmisc::printlog 'info', "purge $table $col $purge =",
+        $db->do( "DELETE FROM $tq$table$tq WHERE $col < " . int( time - $purge ) );
     }
     $db->optimize() unless $config{'no_auto_optimize'};
   } elsif ( $arg eq 'install' ) {
@@ -101,7 +123,7 @@ for my $arg (@ARGV) {
     #$db->do( "DROP TABLE $_")       for qw(queries_top_string_daily queries_top_tth_daily results_top_daily);
     local $db->{'auto_install'} = 0;
     local $db->{'error_sleep'}  = 0;
-    my ( $tq, $rq, $vq ) = $db->quotes();
+    #my ( $tq, $rq, $vq ) = $db->quotes();
 
 =old 
     $db->do( "DROP TABLE ${_}d")    for qw(queries_top_string_ queries_top_tth_ results_top_);
@@ -118,10 +140,17 @@ for my $arg (@ARGV) {
         for qw(queries_top_string_ queries_top_tth_ results_top_);
     }
 =cut
-
   } elsif ( $arg eq 'stat' ) {
-    $ARGV[$n] = undef;
+    $ARGV[$n]             = undef;
+    $db->{'auto_repair'}  = 1;
+    $db->{'force_repair'} = 1;
     $db->table_stat();
+  } elsif ( $arg eq 'check' ) {
+    $ARGV[$n]             = undef;
+    $db->{'auto_repair'}  = 1;
+    $db->{'force_repair'} = 1;
+    #$db->check();
+    $db->check_data();
   }
 }
 our %work;
@@ -132,35 +161,35 @@ sub close_all {
   $db->disconnect();
   $_->destroy() for @dc;
   psmisc::caller_trace(5);
-  printlog "bye close_all";
+  psmisc::printlog "bye close_all";
   exit;
 }
 sub flush_all { $db->flush_insert(); }
 
 sub print_info {
-  printlog( 'info', "queue len=", scalar @{ $work{'toask'} || [] }, " first hits=", $work{'ask'}{ $work{'toask'}[0] } );
+  psmisc::printlog( 'info', "queue len=", scalar @{ $work{'toask'} || [] }, " first hits=", $work{'ask'}{ $work{'toask'}[0] } );
   local @_ = grep { $_ and $_->active() } @dc;
-  printlog 'info', 'active hubs:', map { $_->{'host'} . ':' . $_->{'status'} } @_;
-  printlog 'info', 'hashes:',      map { $_ . '=' . scalar %{ $work{$_} || {} } } qw(ask asked ask_db);
-  printlog 'info', 'stat:',        map { $_ . '=' . $work{'stat'}{$_} } keys %{ $work{'stat'} || {} };
+  psmisc::printlog 'info', 'active hubs:', map { $_->{'host'} . ':' . $_->{'status'} } @_;
+  psmisc::printlog 'info', 'hashes:',      map { $_ . '=' . scalar %{ $work{$_} || {} } } qw(ask asked ask_db);
+  psmisc::printlog 'info', 'stat:',        map { $_ . '=' . $work{'stat'}{$_} } keys %{ $work{'stat'} || {} };
   #psmisc::file_rewrite(    'dumper',    Dumper [      'work' => \%work,      'db'   => $db,      'dc'   => \@dc,    ]  );
   if ( $^O =~ /win/i ) {
     our $__hup_time__;
-    printlog( 'info', 'doubleclose, bye' ), exit if time - $__hup_time__ < 2;
+    psmisc::printlog( 'info', 'doubleclose, bye' ), exit if time - $__hup_time__ < 2;
     $__hup_time__ = time;
   }
 }
-$SIG{INT} = $SIG{__DIE__} = \&close_all;
-$SIG{HUP}      = $^O =~ /win/i ? \&print_info : \&flush_all;
-$SIG{INFO}     = \&print_info;
-$SIG{__WARN__} = sub {
-  printlog( 'warn', $!, $@, @_ );
+local $SIG{INT} = $SIG{__DIE__} = \&close_all;
+local $SIG{HUP}      = $^O =~ /win/i ? \&print_info : \&flush_all;
+local $SIG{INFO}     = \&print_info;
+local $SIG{__WARN__} = sub {
+  psmisc::printlog( 'warn', $!, $@, @_ );
   #printlog( 'die', 'caller', $_, caller($_) ) for ( 0 .. 15 );
   psmisc::caller_trace(15);
 };
-$SIG{__DIE__} = sub {
-  printlog( 'die', $!, $@, @_ );
-  printlog( 'die', 'caller', $_, caller($_) ) for ( 0 .. 15 );
+local $SIG{__DIE__} = sub {
+  psmisc::printlog( 'die', $!, $@, @_ );
+  #printlog( 'die', 'caller', $_, caller($_) ) for ( 0 .. 15 );
   psmisc::caller_trace(5);
 };
 for ( grep { length $_ } @ARGV ) {
@@ -172,6 +201,7 @@ for ( grep { length $_ } @ARGV ) {
     my $hub = $_;
     ++$work{'hubs'}{$hub};
     my $dc = Net::DirectConnect->new(
+      #modules            => ['filelist'],
       'host'      => $hub,
       'Nick'      => 'dcstat',
       'sharesize' => 40_000_000_000 + int( rand 10_000_000_000 ),
@@ -188,9 +218,10 @@ for ( grep { length $_ } @ARGV ) {
       #'auto_connect' => 0,
       'reconnects' => 500,
       'handler'    => {
-        'Search_parse_aft' => sub {
+        #'Search_parse_aft' => sub {
+        'Search' => sub {
           my $dc = shift;
-          printlog 'sch', Dumper @_ if $dc->{adc};
+          #printlog 'sch', Dumper @_ if $dc->{adc};
           my $who    = shift if $dc->{adc};
           my $search = shift if $dc->{nmdc};
           my $s = $_[0] || {};
@@ -223,25 +254,26 @@ for ( grep { length $_ } @ARGV ) {
                   grep { $work{'ask'}{$_} >= $config{'hit_to_ask'} and !exists $work{'asked'}{$_} } keys %{ $work{'ask'} }
                 )
               ];
-              printlog( 'warn', "reasking" ), $work{'toask'} = [ (
+              psmisc::printlog( 'warn', "reasking" ), $work{'toask'} = [ (
                   sort { $work{'ask'}{$b} <=> $work{'ask'}{$a} } grep {
-                    $work{'ask'}{$_} >= $config{'hit_to_ask'}
+                          $work{'ask'}{$_} >= $config{'hit_to_ask'}
                       and $work{'asked'}{$_}
                       and $work{'asked'}{$_} + $config{'ask_retry'} < $time
                     } keys %{ $work{'ask'} }
                 )
                 ]
                 unless @{ $work{'toask'} };
-              printlog( 'info', "queue len=", scalar @{ $work{'toask'} }, " first hits=", $work{'ask'}{ $work{'toask'}[0] } );
+              psmisc::printlog( 'info', "queue len=", scalar @{ $work{'toask'} },
+                " first hits=", $work{'ask'}{ $work{'toask'}[0] } );
             }
           );
           psmisc::schedule(
             [ 3600, 3600 ],
             our $hashes_cleaner_ ||= sub {
               my $min = scalar keys %{ $work{'hubs'} || {} };
-              printlog 'info', "queue clear min[$min] now", scalar %{ $work{'ask'} || {} };
+              psmisc::printlog 'info', "queue clear min[$min] now", scalar %{ $work{'ask'} || {} };
               delete $work{'ask'}{$_} for grep { $work{'ask'}{$_} < $min } keys %{ $work{'ask'} || {} };
-              printlog 'info', "queue clear ok now", scalar %{ $work{'ask'} || {} };
+              psmisc::printlog 'info', "queue clear ok now", scalar %{ $work{'ask'} || {} };
             }
           );
           psmisc::schedule(
@@ -264,7 +296,7 @@ for ( grep { length $_ } @ARGV ) {
               }
               if ( !$dc->{'search_todo'} ) {
                 $work{'asked'}{$q} = int time;
-                printlog( 'info', "search", $q, 'on', $dc->{'host'} );
+                psmisc::printlog( 'info', "search", $q, 'on', $dc->{'host'} );
                 $dc->search($q);
               } else {
                 unshift @{ $work{'toask'} }, $q;
@@ -273,15 +305,18 @@ for ( grep { length $_ } @ARGV ) {
             $dc
           );
         },
-        'SR_parse_aft' => sub {
+        #'SR_parse_aft' => sub {
+        'SR' => sub {
           my $dc = shift;
           my %s = %{ $_[1] || return };
+          #printlog('SR recieved');
           $db->insert_hash( 'results', \%s );
           ++$work{'stat'}{'SR'};
         },
         'chatline' => sub {
           my $dc = shift;
-          printlog( 'chatline', @_ );
+          #psmisc::printlog( 'chatline', @_ );
+          $dc->say( 'chatline', @_ );
           my %s;
           ( $s{nick}, $s{string} ) = $_[0] =~
             #/^<([^>]+)> (.+)$/s;
@@ -289,12 +324,13 @@ for ( grep { length $_ } @ARGV ) {
           if ( $s{nick} and $s{string} ) {
             $db->insert_hash( 'chat', { %s, 'time' => int(time), 'hub' => $dc->{'hub_name'}, } );
           } else {
-            printlog( 'err', 'wtf chat', @_ );
+            psmisc::printlog( 'err', 'wtf chat', @_ );
           }
         },
         'welcome' => sub {
           my $dc = shift;
-          printlog( 'welcome', @_ );
+          #psmisc::printlog( 'welcome', @_ );
+          $dc->say( 'welcome', @_ );
         },
         'MyINFO' => sub {
           my $dc = shift;
@@ -307,7 +343,7 @@ for ( grep { length $_ } @ARGV ) {
               'size'   => $dc->{'NickList'}{$_}{'sharesize'},
               'ip'     => $dc->{'NickList'}{$_}{'ip'},
               'port'   => $dc->{'NickList'}{$_}{'port'},
-              'info'   => Data::Dumper->new( [ $dc->{'NickList'}{$_} ] )->Indent(0)->Terse(1)->Purity(1)->Dump(),
+              'info'   => Data::Dumper->new( [ $dc->{'NickList'}{$_} ] )->Indent(0)->Pair('=>')->Terse(1)->Purity(1)->Dump(),
               'online' => int time
             }
           );
@@ -324,7 +360,7 @@ for ( grep { length $_ } @ARGV ) {
               'size'   => $dc->{'NickList'}{$_}{'sharesize'},
               'ip'     => $dc->{'NickList'}{$_}{'ip'},
               'port'   => $dc->{'NickList'}{$_}{'port'},
-              'info'   => Data::Dumper->new( [ $dc->{'NickList'}{$_} ] )->Indent(0)->Terse(1)->Purity(1)->Dump,
+              'info'   => Data::Dumper->new( [ $dc->{'NickList'}{$_} ] )->Indent(0)->Pair('=>')->Terse(1)->Purity(1)->Dump(),
               'online' => 0
             }
           );
@@ -345,7 +381,7 @@ for ( grep { length $_ } @ARGV ) {
               'size' => $params->{SS},
               'ip'   => $params->{I4},
               'port' => $params->{U4},
-              'info' => Data::Dumper->new( [$params] )->Indent(0)->Terse(1)->Purity(1)->Dump(),
+              'info' => Data::Dumper->new( [$params] )->Indent(0)->Pair('=>')->Terse(1)->Purity(1)->Dump(),
               #maybe full from peers ?
               'online' => int time
             }
@@ -356,7 +392,7 @@ for ( grep { length $_ } @ARGV ) {
         'QUI' => sub {
           my $dc = shift;
           local $_ = $_[0];
-          printlog 'qui', Dumper @_;
+          #printlog 'qui', Dumper @_;
 
 =c
           $db->insert_hash(
@@ -372,18 +408,31 @@ for ( grep { length $_ } @ARGV ) {
             }
           );
 =cut
-
           ++$work{'stat'}{'QUI'};
         },
         'RES' => sub {
           #$db->insert_hash( 'results', \%s );
+          psmisc::printlog 'RES:', Dumper @_;
           ++$work{'stat'}{'RES'};
         },
+        #'FSCH' => sub {
+        #	printlog 'FSCH:', Dumper @_;
+        #  #$db->insert_hash( 'results', \%s );
+        #  ++$work{'stat'}{'FSCH'};
+        #},
+        'MSG' => sub {
+          my $dc = shift;
+          #$db->insert_hash( 'results', \%s );
+          #psmisc::printlog 'MSG:', Dumper @_;
+          $dc->say( 'MSG', @_ );
+          ++$work{'stat'}{'MSG'};
+        },
       },
       %config,
     );
     #$dc->connect($hub);
-    $dc->{'handler'}{'SCH_parse_aft'} = $dc->{'handler'}{'Search_parse_aft'};
+    #$dc->{'handler'}{'SCH_parse_aft'} = $dc->{'handler'}{'Search_parse_aft'};
+    $dc->{'handler'}{'SCH'} = $dc->{'handler'}{'Search'};
 
 =no    
 	$dc->{'clients'}{'listener_http'}{'handler'}{''} = sub {
@@ -394,7 +443,6 @@ for ( grep { length $_ } @ARGV ) {
       $dc->destroy();
     };
 =cut	
-
     push @dc, $dc;
     $_->work() for @dc;
   }
@@ -418,7 +466,7 @@ while ( my @dca = grep { $_ and $_->active() } @dc ) {
         $dc->work(1);
         if   ( $dc->{nmdc} ) { $share += $dc->{'NickList'}{$_}{'sharesize'} for @users; }
         else                 { $share += $dc->{'peers_sid'}{$_}{INF}{'SS'}  for @users; }
-        printlog 'info', "hubsize $dc->{'hub_name'}: bytes = $share users=", scalar @users;
+        psmisc::printlog 'info', "hubsize $dc->{'hub_name'}: bytes = $share users=", scalar @users;
         $db->insert_hash( 'hubs', { 'time' => $time, 'hub' => $dc->{'hub_name'}, 'size' => $share, 'users' => scalar @users } )
           if $share;
       }
@@ -427,24 +475,22 @@ while ( my @dca = grep { $_ and $_->active() } @dc ) {
     ,
     @dc
   );
-  psmisc::schedule( [ 300, 60 * 40 ], our $hubrunhour_ ||= sub { psmisc::startme('calch'); } ),
+  psmisc::schedule( [ 300, 60 * 19 ], our $hubrunhour_ ||= sub { psmisc::startme('calch'); } ),
     psmisc::schedule( [ 600, 60 * 60 * 6 ], our $hubrunrare_ ||= sub { psmisc::startme('calcr'); } )
     if $config{'use_slow'};
 #psmisc::schedule( [ 60 * 3, 60 * 60 * 24 ], our $hubrunoptimize_ ||= sub { psmisc::startme('calcr'); } )    if $config{'auto_optimize'};
   psmisc::schedule( [ 900, 86400 ], $config{'purge'} / 10, our $hubrunpurge_ ||= sub { psmisc::startme('purge'); } );
-
-=z
-   psmisc::schedule(
+  #=z
+  psmisc::schedule(
     [ 10, 100 ],
     our $dump_sub__ ||= sub {
       print "Writing dump\n";
-      psmisc::file_rewrite( 'dump', Dumper @dc);
+      psmisc::file_rewrite( 'dump', Dumper @dc );
     }
-  );
-=cut
-
+  ) if $config{'debug'};
+  #=cut
 }
-printlog 'dev', map { $_->{'host'} . ":" . $_->{'status'} } @dc;
+psmisc::printlog 'dev', map { $_->{'host'} . ":" . $_->{'status'} } @dc;
 #psmisc::caller_trace(20);
 $_->destroy() for @dc;
-printlog 'info', 'bye', times;
+psmisc::printlog 'info', 'bye', times;
