@@ -1,8 +1,10 @@
-#$Id: DirectConnect.pm 631 2010-06-15 11:19:59Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect.pm $
+#$Id: DirectConnect.pm 686 2010-12-16 00:02:50Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect.pm $
 package Net::DirectConnect;
 use strict;
-our $VERSION = '0.06'; # . '_' . ( split( ' ', '$Revision: 631 $' ) )[1];
+our $VERSION = '0.07';    # . '_' . ( split( ' ', '$Revision: 686 $' ) )[1];
 no warnings qw(uninitialized);
+use utf8;
+use Encode;
 use Socket;
 use IO::Socket;
 use IO::Select;
@@ -22,12 +24,9 @@ sub float {    #v1
 
 sub send_udp ($$;@) {
   my $self = shift if ref $_[0];
-  #$self->log('dcdev', "sending UDP0:", Dumper \@_);
   my $host = shift;
   $host =~ s/:(\d+)$//;
   my $port = shift;
-  #$port = $1 || shift ;
-  #$port = shift ;
   $port ||= $1;
   $self->log( 'dcdev', "sending UDP to [$host]:[$port] = [$_[0]]" );
   my $opt = $_[1] || {};
@@ -48,7 +47,6 @@ sub send_udp ($$;@) {
     )
     )
   {
-    #$self->log('dcdev', "sending UDP to [$host]:[$port] RES=" ,$s->send($_[0]));
     $s->send( $_[0] );
     $self->{bytes_send} += length $_[0];
     $s->close();
@@ -58,21 +56,16 @@ sub send_udp ($$;@) {
 }
 
 sub schedule($$;@)
-{    #$Id: DirectConnect.pm 631 2010-06-15 11:19:59Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect.pm $
+{    #$Id: DirectConnect.pm 686 2010-12-16 00:02:50Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect.pm $
   our %schedule;
-  #for (1..100000000) { psmisc::schedule(10, our $my_every_10sec_sub__ ||= sub { print "every 10 sec"})};
   my ( $every, $func ) = ( shift, shift );
   my $p;
   ( $p->{'wait'}, $p->{'every'}, $p->{'runs'}, $p->{'cond'}, $p->{'id'} ) = @$every if ref $every eq 'ARRAY';
   $p = $every if ref $every eq 'HASH';
   $p->{'every'} ||= $every if !ref $every;
   $p->{'id'} ||= join ';', caller;
-  #$p->{'id'} ||= $func;
   $schedule{ $p->{'id'} }{'func'} = $func if !$schedule{ $p->{'id'} }{'func'} or $p->{'update'};
   $schedule{ $p->{'id'} }{'last'} = time - $p->{'every'} + $p->{'wait'} if $p->{'wait'} and !$schedule{ $p->{'id'} }{'last'};
-  #printlog 'everyS1', $schedule{ $p->{'id'} }{'func'}, $func,Dumper([ $every,  $schedule{ $p->{'id'} } ], $p), caller;
-  #printlog 'everyS4', Dumper([ $every, $func, $p, $schedule{$func} ]), $func;
-  #printlog('dev','everyR', Dumper($p, $schedule{$p->{'id'}} ), time, $func ),
   $schedule{ $p->{'id'} }{'func'}->(@_), $schedule{ $p->{'id'} }{'last'} = time
     if ( $schedule{ $p->{'id'} }{'last'} + $p->{'every'} < time )
     and ( !$p->{'runs'} or $schedule{ $p->{'id'} }{'runs'}++ < $p->{'runs'} )
@@ -80,73 +73,44 @@ sub schedule($$;@)
     and ref $schedule{ $p->{'id'} }{'func'} eq 'CODE';
 }
 
+sub use_try ($;@) {
+  my $self = shift if ref $_[0];
+  our %tried;
+  ( my $path = ( my $module = shift ) . '.pm' ) =~ s{::}{/}g;
+  return $tried{$module} if exists $tried{$module};
+  $tried{$module} = ( $INC{$path} or eval 'use ' . $module . ' qw(' . ( join ' ', @_ ) . ');1;' and $INC{$path} );
+}
+
 sub module_load {
   my $self = shift if ref $_[0];
   local $_ = shift;
-  #for (@_) {
   return unless length $_;
-  #%self
-  #my $module = __PACKAGE__ . '::' . $self->{'module'};
   my $module = __PACKAGE__ . '::' . $_;
-  #$self->log( 'dev', 'try load module', $_, $module, );
-  #eval "use $module; $module\->init(\$self);";
   eval "use $module;";
-  $self->log( 'err', 'cant load', $module, $@ ) if $@;
-  #$self->log( 'dev', 'try new', $module );
+  $self->log( 'err', 'cant load', $module, $@ ), return if $@;
   eval "$module\::new(\$self, \@_);";    #, \@param
-  $self->log( 'err', 'cant new', $module, $@ ) if $@;
-  #$self->log( 'dev', 'try init', $module );
+  $self->log( 'err', 'cant new', $module, $@ ), return if $@;
   eval "$module\::init(\$self, \@_);";    #, \@param
-  $self->log( 'err', 'cant init', $module, $@ ) if $@;
+  $self->log( 'err', 'cant init', $module, $@ ), return if $@;
   $self->log( 'dev', 'loaded  module', $_, $module, );
-  #}
+  1;
 }
 
 sub new {
   my $class = shift;
-  #psmisc::printlog( 'dev', __PACKAGE__, 'new', $class, 'refc=',ref $class, 'next=', $_[0], 'refnext=', ref$_[0], scalar @_ );
-  #print 'NEW', __PACKAGE__, "\n";
-  #my @param = @_;
-  my $self = {};
+  my $self  = {};
   if ( ref $class eq __PACKAGE__ ) { $self = $class; }
-  else {
-    #$self  =   {
-    bless( $self, $class ) unless ref $class;
-    #psmisc::printlog('dev', 'blessed', "self=$self, class=$class package=", __PACKAGE__);
-    #print (  'init ', 'class ', $class, __LINE__,,"[  ]\n\n");
-    #$self->log( 'dev', 'prefunc', $self, $class );
-  }
+  else                             { bless( $self, $class ) unless ref $class; }
   local %_ = (
-    'Listen'      => 10,
-    'Timeout'     => 5,
-    'myport'      => 412,                                                   #first try
-    'myport_base' => 40000, 'myport_random' => 1000, 'myport_tries' => 5,
-    #http://www.dcpp.net/wiki/index.php/%24MyINFO
-    'description' => 'perl ' . __PACKAGE__ . ' bot',
-    #====MOVE TO NMDC
-    'connection' => 'LAN(T3)',
-    #NMDC1: 28.8Kbps, 33.6Kbps, 56Kbps, Satellite, ISDN, DSL, Cable, LAN(T1), LAN(T3)
-    #NMDC2: Modem, DSL, Cable, Satellite, LAN(T1), LAN(T3)
-    'flag' => '1',                                                          # User status as ascii char (byte)
-    #1 normal
-    #2, 3 away
-    #4, 5 server               The server icon is used when the client has
-    #6, 7 server away          uptime > 2 hours, > 2 GB shared, upload > 200 MB.
-    #8, 9 fireball             The fireball icon is used when the client
-    #10, 11 fireball away      has had an upload > 100 kB/s.
-    'email' => 'billgates@microsoft.com', 'sharesize' => 10 * 1024 * 1024 * 1024 + int rand( 1024 * 1024 ),    #10GB
-    'client' => 'perl',    #'dcp++',                                                              #++: indicates the client
-    #'protocol' => 'nmdc',    # or 'adc'
-    'V' => $VERSION,       #. '_' . ( split( ' ', '$Revision: 631 $' ) )[1],    #V: tells you the version number
-    #'M' => 'A',      #M: tells if the user is in active (A), passive (P), or SOCKS5 (5) mode
-    'H' => '0/1/0'
-    , #H: tells how many hubs the user is on and what is his status on the hubs. The first number means a normal user, second means VIP/registered hubs and the last one operator hubs (separated by the forward slash ['/']).
-    'S' => '3',      #S: tells the number of slots user has opened
-    'O' => undef,    #O: shows the value of the "Automatically open slot if speed is below xx KiB/s" setting, if non-zero
-    'lock'     => 'EXTENDEDPROTOCOLABCABCABCABCABCABC Pk=DCPLUSPLUS0.668ABCABC',
-    'cmd_sep'  => ' ',
-    'no_print' => { map { $_ => 1 } qw(Search Quit MyINFO Hello SR UserCommand) },
-    'log'      => sub (@) {
+    'Listen'        => 10,
+    'Timeout'       => 5,
+    'myport'        => 412,                                                               #first try
+    'myport_base'   => 40000,
+    'myport_random' => 1000,
+    'myport_tries'  => 5,
+    'cmd_sep'       => ' ',
+    'no_print'      => { map { $_ => 1 } qw(Search Quit MyINFO Hello SR UserCommand) },
+    'log'           => sub (@) {
       my $self = ref $_[0] ? shift() : {};
       if ( ref $self->{'parent'}{'log'} eq 'CODE' ) { return $self->{'parent'}->log( "[$self->{'number'}]", @_ ); }
       print( join( ' ', "[$self->{'number'}]", @_ ), "\n" );
@@ -165,30 +129,32 @@ sub new {
     #( $^O eq 'MSWin32' ? () : ( 'nonblocking' => 1 ) ),
     'nonblocking' => 1,
     'informative' => [qw(number peernick status host port filebytes filetotal proxy bytes_send bytes_recv)],    # sharesize
-    'informative_hash' => [qw(clients)],    #NickList IpList PortList
-    #'disconnect_recursive' => 1,
-    'reconnect_sleep' => 5, 'partial_ext' => '.partial', 'file_send_by' => 1024 * 1024,    #1024 * 64,
+    'informative_hash' => [qw(clients)],                                                    #NickList IpList PortList
+                                                                                            #'disconnect_recursive' => 1,
+    'reconnect_sleep'  => 5, 'partial_ext' => '.partial', 'file_send_by' => 1024 * 1024,    #1024 * 64,
     'local_mask_rfc' => [qw(10 172.[123]\d 192\.168)], 'status' => 'disconnected', time_start => time,
     #'peers' => {},
     #'partial_prefix' => './partial/',
     #ADC
     #number => ++$global{'total'},
     #};
-    charset_fs => ( $^O eq 'MSWin32' ? 'cp1251' : $^O eq 'freebsd' ? 'koi8r' : 'utf8' ),
+    charset_fs      => ( $^O eq 'MSWin32' ? 'cp1251' : $^O eq 'freebsd' ? 'koi8r' : 'utf8' ),
+    charset_console => ( $^O eq 'MSWin32' ? 'cp866'  : $^O eq 'freebsd' ? 'koi8r' : 'utf8' ),
+    charset_protocol => 'utf8',
+    charset_internal => 'utf8',
   );
   $self->{$_} ||= $_{$_} for keys %_;
   local %_ = @_;
   $self->{$_} = $_{$_} for keys %_;
+  #$self->log("charset_console=$self->{charset_console} charset_fs=$self->{charset_fs}");
   #psmisc::printlog('dev', 'init0', Dumper $self);
-
-
   #psmisc::printlog('dev', 'func');
   $self->func();    #@param
   eval { $self->{'recv_flags'} = MSG_DONTWAIT; } unless $^O =~ /win/i;
   $self->{'recv_flags'} ||= 0;
   #psmisc::printlog('dev', 'init');
   $self->init();    #@param
-  #}
+                    #}
   $self->{'number'} ||= ++$global{'total'};
   ++$global{'count'};
   $self->{activity} = time;
@@ -207,6 +173,17 @@ sub new {
     #for keys
     #$self->{$_} = $_{$_} for keys %_;
     #$self->log( 'init00', $self, "h=$self->{'host'}", 'p=', $self->{'protocol'}, 'm=', $self->{'module'} );
+    if ( $self->{'host'} eq 'broadcast' or $self->{'host'} =~ /^255\./ ) {
+      $self->{'protocol'} ||= 'adc';
+      $self->{'auto_listen'}           = 1;
+      $self->{'Proto'}                 = 'udp';
+      $self->{'sockopts'}{'Broadcast'} = 1;
+      $self->{'sockopts'}{'ReuseAddr'} = 1;
+      $self->{'host'}                  = inet_ntoa(INADDR_BROADCAST) if $self->{'host'} !~ /^255\./;
+      #$self->{'port'},
+      #$self->log( 'dev',  "send to", );
+      $self->{'broadcast'} = 1;
+    }
     if ( !$self->{'module'} and !$self->{'protocol'} and $self->{'host'} ) {
       #$self->log( 'proto0 ', $1);
       my $p = lc $1 if $self->{'host'} =~ m{^(.+?)://};
@@ -217,7 +194,7 @@ sub new {
       #$self->log( 'proto ', $self->{'protocol'} );
     }
     $self->{'module'} ||= $self->{'protocol'};
-    if ( $self->{'module'} eq 'nmdc' ) { $self->{'module'} = $self->{'hub'} ? 'hubcli' : 'clihub'; }
+    if ( $self->{'module'} eq 'nmdc' ) { $self->{'module'} = [ 'nmdc', ( $self->{'hub'} ? 'hubcli' : 'clihub' ) ]; }
     #$self->log( 'module load', $self->{'module'});
     #if ( $self->{'module'} ) {
   }
@@ -228,13 +205,17 @@ sub new {
     push @modules, keys %{ $self->{$_} } if ref $self->{$_} eq 'HASH';
     push @modules, split /[;,\s]/, $self->{$_} unless ref $self->{$_};
   }
-  $self->module_load( $_, ) for (@modules);
+  #$self->log( 'modules load', @modules);
+  $self->module_load($_) for @modules;
   #@param
   #}
+  $self->{charset_chat} ||= $self->{charset_protocol};
   $self->protocol_init();
   #$self->log( 'dev', $self, 'new inited', "MT:$self->{'message_type'}", 'autolisten=', $self->{'auto_listen'} );
-  if ( $self->{'auto_listen'} ) { $self->listen(); }
-  elsif ( $self->{'auto_connect'} ) {
+  if ( $self->{'auto_listen'} ) {
+    $self->listen();
+    $self->cmd('connect_aft') if $self->{'broadcast'};
+  } elsif ( $self->{'auto_connect'} ) {
     #$self->log( $self, 'new inited', "auto_connect MT:$self->{'message_type'}", ' with' );
     $self->connect();
     #$self->work();
@@ -247,7 +228,7 @@ sub new {
     #$self->log( $self, '', "auto_work ", $self->active() );
     while ( $self->active() ) {
       $self->work();    #forever
-      #$self->{'auto_work'}->($self) if ref $self->{'auto_work'} eq 'CODE';
+                        #$self->{'auto_work'}->($self) if ref $self->{'auto_work'} eq 'CODE';
     }
     $self->disconnect();
   }
@@ -257,7 +238,6 @@ sub new {
 
 sub log(@) {
   my $self = shift;
-  #print 'LOG0 ', $self->{'log'}, '; ';
   return $self->{'log'}->( $self, @_ ) if ref $self->{'log'} eq 'CODE';
   print( join( ' ', "[$self->{'number'}]", @_ ), "\n" );
 }
@@ -300,7 +280,8 @@ sub cmd {
     @ret = $self->cmd_adc( $dst, $cmd, @_ );
   } else {
     $self->log(
-      'info', "UNKNOWN CMD:[$cmd]{@_} : please add \$dc->{'cmd'}{'$cmd'} = sub { ... };",
+      'info',  "UNKNOWN CMD:[$cmd]{@_} : please add \$dc->{'cmd'}{'$cmd'} = sub { ... };",
+      "self=", ref $self,
       #Dumper $self->{'cmd'},
       $self->{'parse'}
     );
@@ -330,7 +311,7 @@ sub AUTOLOAD {
 sub DESTROY {
   my $self = shift;
   #print "DESTROYing $self->{number}\n";
-  $self->log( 'dev', 'DESTROYing' );
+  #$self->log( 'dev', 'DESTROYing' );
   $self->destroy();
   --$global{'count'};
 }
@@ -507,7 +488,7 @@ sub func {
         'LocalPort' => $self->{'myport'},
         'Proto'     => $self->{'Proto'} || 'tcp',
         ( $self->{'Proto'} ne 'udp' ? ( 'Listen' => $self->{'Listen'} ) : () ),
-        ( $self->{'nonblocking'} ? ( 'Blocking' => 0 ) : () ), %{ $self->{'sockopts'} or {} },
+        ( $self->{'nonblocking'} ? ( 'Blocking' => 0 ) : () ), %{ $self->{'sockopts'} || {} },
       );
       $self->select_add(), last if $self->{'socket'};
       $self->log( 'err', "listen $self->{'myport'} socket error: $@" ), $self->myport_generate(1), unless $self->{'socket'};
@@ -609,7 +590,8 @@ sub func {
     }
     $self->log( 'dev', "SOCKERR", $client, $self->{'socket'}, $self->{'select'} ) if $client ne $self->{'socket'};
     $self->{'databuf'} = '';
-    if ( !defined( my $r = $client->recv( $self->{'databuf'}, POSIX::BUFSIZ, $self->{'recv_flags'} ) )
+    #my $r;
+    if ( !defined( $self->{'recv_addr'} = $client->recv( $self->{'databuf'}, POSIX::BUFSIZ, $self->{'recv_flags'} ) )
       or !length( $self->{'databuf'} ) )
     {
       #TODO not here
@@ -642,6 +624,7 @@ sub func {
         #$self->log( 'dcdbg',"[$self->{'number'}] dev cycle ",length $_," [$_]", );
         last unless length $_ and length $self->{'cmd_aft'};
         next unless length;
+        $self->get_peer_addr_recv() if $self->{'broadcast'};
         $self->parser($_);
         last if ( $self->{'filehandle'} );
       }
@@ -667,7 +650,7 @@ sub func {
 #schedule(10, sub {        $self->log( 'dev', 'DC::select', 'aft' , Dumper ($recv, $send, $exeption), 'from', $self->{'select'}->handles() ,    'and ', $self->{'select_send'}->handles());        });
     for (@$exeption) { $self->log( 'err', 'exeption', $_, $self->{sockets}{$_}{number} ); }
     for (@$recv) {
-      $self->log( 'err', 'no object for recv handle', $_, ), next, unless $self->{sockets}{$_};
+      $self->log( 'err', 'no object for recv handle', $_, ), next, if !$self->{sockets}{$_} or !ref $self->{sockets}{$_};
       $self->{sockets}{$_}->recv($_);
     }
     for (@$send) {
@@ -675,7 +658,6 @@ sub func {
       #$self->log( 'dev', 'can_send', $_, $self->{sockets}{$_}{number} );
       if ( $self->{sockets}{$_}{'filehandle_send'} ) { $self->{sockets}{$_}->file_send_part(); }
     }
-
     #if ( $self->{'filehandle_send'} ) { $self->file_send_part(); }
     #$self->{'recv_runned'}{ $self->{'number'} } = undef;
     return $ret;
@@ -789,6 +771,27 @@ sub func {
         }
         $self->{$_}->($self) for grep { ref $self->{$_} eq 'CODE' } qw(worker auto_work);
         #$self->log('dev', 'work exit',      );
+        if ( $self->{'queue_download'} and @{ $self->{'queue_download'} } ) {
+          my $file = shift @{ $self->{'queue_download'} };
+          $self->search($file);
+        }
+        #=todo
+        for my $tth ( grep { keys %{ $self->{'want_download'}{$_} } } keys %{ $self->{'want_download'} } ) {
+          if ( my ($from) = ( grep { $_->{slotsopen} or $_->{SL} } values %{ $self->{'want_download'}{$tth} } ) ) {
+            my $filename = $from->{FN};
+            $filename =~ s{^.*[/\\]}{}g;
+            $self->log( "selected [$filename] from", Dumper $from);
+            my $dst = $self->{'get_dir'} . $filename;
+            my $size = $from->{size} || $from->{SI};
+            unless ( -e $dst and ( !$size or -s $dst == $size ) ) {
+              $self->get( $from->{nick} || $from->{NI}, 'TTH/' . $tth, $dst );
+              delete $self->{'want_download'}{$tth};    #dont!
+              last;
+            }
+            #$work{'tthfrom'}{$s{tth}}
+          }
+        }
+        #=cut
       }
     );
     return $self->wait_sleep(@params) if @params;
@@ -797,9 +800,11 @@ sub func {
   $self->{'parser'} ||= sub {
     my $self = shift;
     for ( local @_ = @_ ) {
-      $self->log( 'dcdmp', "rawrcv:", $_ );
+      $self->log( 'dcdmp', "rawrcv[" . ( $self->{'recv_host'} || $self->{'host'} ) . "]:", $_ );
       my ( $dst, $cmd, @param );
-      $cmd = ( $self->{'status'} eq 'connected' ? 'chatline' : 'welcome' ) if /^[<*]/;    #farcolorer
+      if (/^[<*]/) {
+        $cmd = ( $self->{'status'} eq 'connected' ? 'chatline' : 'welcome' );
+      }
       s/^\$?([\w\-]+)\s*//, $cmd = $1 unless $cmd;
       if ( $self->{'adc'} ) {
         $cmd =~ s/^([BCDEFHIU])//, $dst = $1;
@@ -808,6 +813,7 @@ sub func {
           #$self->log( 'dcdmp', "P0 $dst$cmd p=",(Dumper \@param));
           #push @{ $param[0] }, shift@param;
           push @{ $param[0] }, splice @param, 1, 1;
+          #$self->log( 'dcdmp', "P0 $dst$cmd p=",(Dumper \@param));
           if ( $dst eq 'F' ) {
             #$self->log( 'dcdmp', 'feature'
             push @{ $param[0] }, splice @param, 1, 1 while $param[1] =~ /^[+\-]/;
@@ -826,12 +832,22 @@ sub func {
       #$self->log( 'dcinf', "UNKNOWN PEERCMD:[$cmd]->($_) : please add \$dc->{'parse'}{'$cmd'} = sub { ... };" ),
       $self->{'parse'}{$cmd} = sub { }, $cmd = ( $self->{'status'} eq 'connected' ? 'chatline' : 'welcome' )
         if $self->{'nmdc'} and !exists $self->{'parse'}{$cmd};
+      if ( $cmd eq 'chatline' or $cmd eq 'welcome' or $cmd eq 'To' ) {
+        #$self->log( 'dev', 'RCV pre encode', ($self->{charset_chat} ), @param, Dumper \@param);
+        #$_ =  Encode::decode(($self->{charset_chat} ), $_) for @param;
+        $_ = Encode::encode $self->{charset_internal}, Encode::decode $self->{charset_chat}, $_ for @param;
+        #$self->log( 'dev', 'RCV postencode', @param, Dumper \@param);
+        #Encode::encode $self->{charset_console},;
+      } else {
+        #$_ =  Encode::encode $self->{charset_internal},
+        #TODO $_ = Encode::decode($self->{charset_protocol}, $_),             for @param;
+      }
       my ( @ret, $ret );
       #$self->log( 'dcinf', "parsing", $cmd, @_ ,'with',$self->{'parse'}{$cmd}, ref $self->{'parse'}{$cmd});
       my @self;
       #@self = $self if $self->{'adc'};
       @self = $self;    #if !$self->{'nmdc'};
-      #$self->handler( @self, $cmd . '_parse_bef_bef', @param );
+                        #$self->handler( @self, $cmd . '_parse_bef_bef', @param );
       $self->handler( @self, $cmd . '_parse_bef', @param );
       if ( ref $self->{'parse'}{$cmd} eq 'CODE' ) {
         if ( !exists $self->{'no_print'}{$cmd} ) {
@@ -861,8 +877,8 @@ sub func {
   $self->{'send'} ||= sub {
     my $self = shift;
     local $_;    # = join( '', @_ );
-    #$self->{bytes_send} += length $_;
-    #eval { $_ = $self->{'socket'}->send( join( '', @_ ) ); } if $self->{'socket'};
+                 #$self->{bytes_send} += length $_;
+                 #eval { $_ = $self->{'socket'}->send( join( '', @_ ) ); } if $self->{'socket'};
     eval { $_ = $self->{'socket'}->send(@_); } if $self->{'socket'};
     $self->{bytes_send} += $_;
     $self->log( 'err', 'send error', $@ ) if $@;
@@ -879,11 +895,15 @@ sub func {
     ++$self->{count_sendcmd}{ $_[0] };
     if ( ( $self->{'sendbuf'} and @_ ) or !@{ $self->{'send_buffer'} || [] } ) { }
     else {
-      $self->log( 'err', "ERROR! no socket to send" ), return unless $self->{'socket'};
-      $self->send( join( '', @{ $self->{'send_buffer'} }, ) );
-      #local $_;
-      #eval { $_ = $self->{'socket'}->send( join( '', @{ $self->{'send_buffer'} }, ) ); };
-      #$self->log( 'err', 'send error', $@ ) if $@;
+      if ( $self->{'broadcast'} ) {
+        $self->send_udp( $self->{'host'}, $self->{'port'}, join( '', @{ $self->{'send_buffer'} }, ) ),;
+      } else {
+        $self->log( 'err', "ERROR! no socket to send" ), return unless $self->{'socket'};
+        $self->send( join( '', @{ $self->{'send_buffer'} }, ) );
+        #local $_;
+        #eval { $_ = $self->{'socket'}->send( join( '', @{ $self->{'send_buffer'} }, ) ); };
+        #$self->log( 'err', 'send error', $@ ) if $@;
+      }
       $self->log( 'dcdmp', "we send [" . join( '', @{ $self->{'send_buffer'} } ) . "]:", $! );
       $self->{'send_buffer'} = [];
       $self->{'sendbuf'}     = 0;
@@ -904,7 +924,7 @@ sub func {
     };
   };
   $self->{'get'} ||= sub {
-    my ( $self, $nick, $file, $as ) = @_;
+    my ( $self, $nick, $file, $as, $from, $to ) = @_;
     my ( $sid, $cid );
     $sid = $nick if $nick =~ /^[A-Z0-9]{4}$/;
     $cid = $nick if $nick =~ /^[A-Z0-9]{39}$/;
@@ -914,7 +934,9 @@ sub func {
     $self->log( 'warn', "file [$_] already exists size = ", -s $_ ) if -e $_;
     #todo by nick
     $self->wait_clients();
-    $self->{'want'}{ $self->{peers}{$cid}{'INF'}{'ID'} || $nick }{$file} = $as || $file || '';
+    #$self->{'want'}{ $self->{peers}{$cid}{'INF'}{'ID'} || $nick }{$file} = $as || $file || '';
+    $self->{'want'}{ $self->{peers}{$cid}{'INF'}{'ID'} || $nick }{$file} =
+      { 'filename' => $file, 'fileas' => $as || $file || '', 'file_recv_to' => $to, 'file_recv_from' => $from };
     $self->log( 'dbg', "getting [$nick] $file as $as" );
     if ( $self->{'adc'} ) {
       #my $token = $self->make_token($nick);
@@ -936,11 +958,12 @@ sub func {
     return if length $self->{'filename'};
     my $peerid = $self->{'peerid'} || $self->{'peernick'};
     #$self->log( 'dcdev','file_select000',$peerid,  $self->{'filename'}, $self->{'fileas'}, Dumper $self->{'want'});
-    for ( keys %{ $self->{'want'}{$peerid} } ) {
-      ( $self->{'filename'}, $self->{'fileas'} ) = ( $_, $self->{'want'}{$peerid}{$_} );
+    for my $file ( keys %{ $self->{'want'}{$peerid} } ) {
+      #( $self->{'filename'}, $self->{'fileas'} ) = ( $_, $self->{'want'}{$peerid}{$_} );
+      $self->{$_} = $self->{'want'}{$peerid}{$file}{$_} for keys %{ $self->{'want'}{$peerid}{$file} };
       #$self->log( 'dcdev', 'file_select1', $self->{'filename'}, $self->{'fileas'} );
-      $self->{'filecurrent'} = $self->{'filename'};
       next unless defined $self->{'filename'};
+      $self->{'filecurrent'} = $self->{'filename'};
       #delete  $self->{'want'}{ $peerid }{$_} ;   $self->{'filecurrent'}
       #$self->{'file_recv_from'}
       #$self->{'fileas'}
@@ -1007,20 +1030,11 @@ sub func {
         ),
         $self->disconnect(), $self->{'status'} = 'destroy', $self->{'file_start_time'} = 0, $self->{'filename'} = '',
         $self->{'fileas'} = '', delete $self->{'want'}{ $self->{'peerid'} }{ $self->{'filecurrent'} },
-        $self->{'filecurrent'} = '', $self->{'file_recv_partial'} = '', $self->{'file_recv_from'} = undef,
+        $self->{'filecurrent'} = '', $self->{'file_recv_partial'} = '',
+        $self->{'file_recv_from'} = $self->{'file_recv_to'} = undef,
         if $self->{'filebytes'} >= $self->{'filetotal'};
     }
   };
-  $self->{'openfile'} ||= sub {
-    my $self = shift;
-    $self->log( 'dcwarn', 'openfile is deprecated, use file_open' );
-    $self->file_open(@_);
-  };
-  $self->{'writefile'} ||= sub {
-    my $self = shift;
-    $self->log( 'dcwarn', 'openfile is deprecated, use file_write' );
-    $self->file_write(@_);
-  };
   $self->{'file_close'} ||= sub {
     my $self = shift;
     #$self->log( 'dcerr', 'file_close', 1);
@@ -1047,7 +1061,7 @@ sub func {
 #$self->{'share_tth'} ||=$self->{'parent'}{'share_tth'};
     if ( $self->{'share_full'}{$file} ) {
       $self->{'share_full'}{$file} =~ tr{\\}{/};
-#$self->log( 'dcdev', 'call send', $self->{'share_full'}{$file}, $start, $size, $as );
+      #$self->log( 'dcdev', 'call send', $self->{'share_full'}{$file}, $start, $size, $as );
       $self->file_send( $self->{'share_full'}{$file}, $start, $size, $as );
     } else {
       $self->log(
@@ -1059,7 +1073,7 @@ sub func {
   };
   $self->{'file_send'} ||= sub {
     my $self = shift;
-#$self->log( 'dcdev', 'file_send', Dumper \@_);
+    #$self->log( 'dcdev', 'file_send', Dumper \@_);
     my ( $file, $start, $size, $as ) = @_;
     $start //= 0;
     $size  //= -s $file;
@@ -1100,53 +1114,47 @@ sub func {
     #my $readed =
     my $sended;
     if ( $INC{'Sys/Sendfile.pm'} ) {    #works
-      #Sys::Sendfile::sendfile fileno($self->{'socket'}), fileno($self->{'filehandle_send'}), $read;
+          #Sys::Sendfile::sendfile fileno($self->{'socket'}), fileno($self->{'filehandle_send'}), $read;
       $self->{'file_send_offset'} += $sended =
         Sys::Sendfile::sendfile( $self->{'socket'}, $self->{'filehandle_send'}, $read, $self->{'file_send_offset'} );
  #);
  #$self->log( 'dev', 'ssendfile0', "$read, offset=$self->{'file_send_offset'}, left=$self->{'file_send_left'} sended=$sended" );
  #$self->{'file_send_offset'} += $sended;
     }
-
-#sux	    
-#    elsif ( $INC{'Sys/Sendfile/FreeBSD.pm'}) {
+#sux
+#elsif ( $INC{'Sys/Sendfile/FreeBSD.pm'}) {
 #use Sys::Sendfile::FreeBSD qw(sendfile);
 #use Errno qw(EINTR EIO :POSIX);
-#    $self->log(      'dev','fsendfile1',  $self->{'file_send_offset'}, $read, 'left', $self->{'file_send_left'}, '=', $sended, 'ff=', fileno($self->{'filehandle_send'}), fileno($self->{'socket'}));
+#$self->log(      'dev','fsendfile1',  $self->{'file_send_offset'}, $read, 'left', $self->{'file_send_left'}, '=', $sended, 'ff=', fileno($self->{'filehandle_send'}), fileno($self->{'socket'}));
 #my $result = sendfile(fileno($self->{'filehandle_send'}), fileno($self->{'socket'}), $self->{'file_send_offset'}, $read, $sended);
 #my $result = sendfile( fileno($self->{'socket'}), fileno($self->{'filehandle_send'}),$self->{'file_send_offset'}, $read, $sended);
-#    $self->log(      'dev','fsendfile1',  $self->{'file_send_offset'}, $read, 'left', $self->{'file_send_left'}, 's=', $sended, 'r=',  $result, $!, 
-#    #Dumper \%!
+#$self->log(      'dev','fsendfile1',  $self->{'file_send_offset'}, $read, 'left', $self->{'file_send_left'}, 's=', $sended, 'r=',  $result, $!,
+##Dumper \%!
 #grep {$!{$_}} keys %!
-#    );
-
+#);
 #}
-
 #sux
-#    elsif ($INC{'IO/AIO.pm'}) {
-
-#    $self->log(      'dev','sendfile0',  $self->{'file_send_offset'}, $read, 'left', $self->{'file_send_left'}, '=', $sended);
+#elsif ($INC{'IO/AIO.pm'}) {
+#$self->log(      'dev','sendfile0',  $self->{'file_send_offset'}, $read, 'left', $self->{'file_send_left'}, '=', $sended);
 #use IO::AIO;
 #$sended = IO::AIO::sendfile(  fileno($self->{'filehandle_send'}), $self->{'socket'}->fileno(),$self->{'file_send_offset'}, $read );
 #$sended = IO::AIO::sendfile(   $self->{'socket'}->fileno(), fileno($self->{'filehandle_send'}),$self->{'file_send_offset'}, $read );
 #$sended = IO::AIO::sendfile(   fileno($self->{'socket'}), fileno($self->{'filehandle_send'}),$self->{'file_send_offset'}, $read );
-# $sended = IO::AIO::sendfile(   $self->{'socket'}, $self->{'filehandle_send'},$self->{'file_send_offset'}, $read );
-# #$self->{'file_send_left'}
-#
-#    $self->log(      'dev','sendfile1',  $self->{'file_send_offset'}, $read, 'left', $self->{'file_send_left'}, '=', $sended);
-#$self->{'file_send_offset'} += $sended;
-#$self->{'file_send_offset'} += $read, $sended = $read,if $sended == 12;
-#} 
-
-
+#$sended = IO::AIO::sendfile(   $self->{'socket'}, $self->{'filehandle_send'},$self->{'file_send_offset'}, $read );
+##$self->{'file_send_left'}
+    #
+    #$self->log(      'dev','sendfile1',  $self->{'file_send_offset'}, $read, 'left', $self->{'file_send_left'}, '=', $sended);
+    #$self->{'file_send_offset'} += $sended;
+    #$self->{'file_send_offset'} += $read, $sended = $read,if $sended == 12;
+    #}
     else {
       read( $self->{'filehandle_send'}, $self->{'file_send_buf'}, $read ),
         $self->{'file_send_offset'} = tell $self->{'filehandle_send'},
         unless length $self->{'file_send_buf'};    #$self->{'file_send_by'};
-      #send $self->{'socket'},
-      #$self->{'socket'}->send( buf, POSIX::BUFSIZ, $self->{'recv_flags'} )
-      #my $sended;
-      #$self->log(      'snd',      length $self->{'file_send_buf'},
+                                                   #send $self->{'socket'},
+                                                   #$self->{'socket'}->send( buf, POSIX::BUFSIZ, $self->{'recv_flags'} )
+                                                   #my $sended;
+                                                   #$self->log(      'snd',      length $self->{'file_send_buf'},
       $sended = $self->send( $self->{'file_send_buf'} );
       #eval {
       #$sended = $self->{'socket'}->send( $self->{'file_send_buf'} );
@@ -1202,7 +1210,7 @@ sub func {
     #$self->{'ADCSND'} =
     sub {
     my $self = shift if ref $_[0];
-#    $self->log(    'cmd_adcSND', Dumper \@_);
+    #$self->log(    'cmd_adcSND', Dumper \@_);
     #my ( $dst, $peerid, $toid ) = @{ shift() };
     if ( $_[0] eq 'file' ) {
       my $file = $_[1];
@@ -1227,9 +1235,15 @@ sub func {
       $self->log( 'dcerr', 'SND', "unknown type", @_ );
     }
     };
-  $self->{'get_peer_addr'} ||= sub {
-    my ($self) = @_;
+  $self->{'download'} ||= sub {
+    my ( $self, $file ) = @_;
+    push @{ $self->{'queue_download'} ||= [] }, $file;
+    $self->{'want_download'}{$file} = {};
+  };
+  $self->{'get_peer_addr'} ||= sub () {
+    my ( $self, $recv ) = @_;
     return unless $self->{'socket'};
+    local @_;
     eval { @_ = unpack_sockaddr_in( getpeername( $self->{'socket'} ) || return ) };
     return unless $_[1];
     return unless $_[1] = inet_ntoa( $_[1] );
@@ -1237,6 +1251,15 @@ sub func {
     $self->{'hostip'} = $_[1], $self->{'host'} ||= $self->{'hostip'} if $_[1];
     return $self->{'hostip'};
   };
+  $self->{'get_peer_addr_recv'} ||= sub (;$) {
+    my ( $self, $recv ) = @_;
+    #return unless $self->{'socket'};
+    $recv ||= $self->{'recv_addr'};
+    ( $self->{'recv_port'}, my $hostn ) = sockaddr_in($recv);
+    $self->{'recv_host'} = gethostbyaddr( $hostn, AF_INET );
+    $self->{'recv_hostip'} = inet_ntoa($hostn);
+    return $self->{'hostip'};
+  };
   $self->{'get_my_addr'} ||= sub {
     my ($self) = @_;
     return unless $self->{'socket'};
@@ -1246,72 +1269,6 @@ sub func {
     #$self->{'log'}->('dev', "MYIP($self->{'myip'}) [$self->{'number'}] SOCKNAME $_[0],$_[1];");
     return $self->{'myip'} ||= $_[1];
   };
-  #http://www.dcpp.net/wiki/index.php/LockToKey :
-  $self->{'lock2key'} ||= sub {
-    my $self = shift if ref $_[0];
-    my ($lock) = @_;
-    #$self->{'log'}->( 'dev', 'making lock from', $lock );
-    my @lock = split( //, $lock );
-    my $i;
-    my @key = ();
-    foreach (@lock) { $_ = ord; }
-    push( @key, $lock[0] ^ 5 );
-    for ( $i = 1 ; $i < @lock ; $i++ ) { push( @key, ( $lock[$i] ^ $lock[ $i - 1 ] ) ); }
-    for ( $i = 0 ; $i < @key ; $i++ ) { $key[$i] = ( ( ( $key[$i] << 4 ) & 240 ) | ( ( $key[$i] >> 4 ) & 15 ) ) & 0xff; }
-    $key[0] = $key[0] ^ $key[ @key - 1 ];
-
-    foreach (@key) {
-      if ( $_ == 0 || $_ == 5 || $_ == 36 || $_ == 96 || $_ == 124 || $_ == 126 ) { $_ = sprintf( '/%%DCN%03i%%/', $_ ); }
-      else                                                                        { $_ = chr; }
-    }
-    return join( '', @key );
-  };
-  $self->{'tag'} ||= sub {
-    my $self = shift;
-    $self->{'client'} . ' ' . join( ',', map $_ . ':' . $self->{$_}, grep defined( $self->{$_} ), qw(V M H S O) );
-  };
-  $self->{'myinfo'} ||= sub {
-    my $self = shift;
-    return
-        $self->{'Nick'} . ' '
-      . $self->{'description'} . '<'
-      . $self->tag() . '>' . '$' . ' ' . '$'
-      . $self->{'connection'}
-      . ( length( $self->{'flag'} ) ? chr( $self->{'flag'} ) : '' ) . '$'
-      . $self->{'email'} . '$'
-      . $self->{'sharesize'} . '$';
-  };
-  $self->{'supports'} ||= sub {
-    my $self = shift;
-    return join ' ', grep $self->{$_}, @{ $self->{'supports_avail'} };
-  };
-  $self->{'supports_parse'} ||= sub {
-    my $self = shift;
-    my ( $str, $save ) = @_;
-    $save->{$_} = 1 for split /\s+/, $str;
-    delete $save->{$_} for grep !length $save->{$_}, keys %$save;
-    return wantarray ? %$save : $save;
-  };
-  $self->{'info_parse'} ||= sub {
-    my $self = shift;
-    my ( $info, $save ) = @_;
-    $save->{'info'} = $info;
-    $save->{'description'} = $1 if $info =~ s/^([^<\$]+)(<|\$)/$2/;
-    ( $save->{'tag'}, $save->{'M'}, $save->{'connection'}, $save->{'email'}, $save->{'sharesize'} ) = split /\s*\$\s*/, $info;
-    $save->{'flag'} = ord($1) if $save->{'connection'} =~ s/([\x00-\x1F])$//e;
-    $self->tag_parse( $save->{'tag'}, $save );
-    delete $save->{$_} for grep !length $save->{$_}, keys %$save;
-    return wantarray ? %$save : $save;
-  };
-  $self->{'tag_parse'} ||= sub {
-    my $self = shift;
-    my ( $tag, $save ) = @_;
-    $save->{'tag'} = $tag;
-    $tag =~ s/(^\s*<\s*)|(\s*>\s*$)//g;
-    $save->{'client'} = $1 if $tag =~ s/^(\S+)\s*//;
-    /(.+):(.+)/, $save->{$1} = $2 for split /,/, $tag;
-    return wantarray ? %$save : $save;
-  };
   $self->{'info'} ||= sub {
     my $self = shift;
     $self->log(
@@ -1359,11 +1316,12 @@ sub func {
   $self->{'cmd_adc'} ||= sub {
     my ( $self, $dst, $cmd ) = ( shift, shift, shift );
     #$self->sendcmd( $dst, $cmd,map {ref $_ eq 'HASH'}@_);
-    #$self->log(    'cmd_adc', Dumper \@_);
+    #$self->log( 'cmd_adc', $dst, $cmd, Dumper \@_ );
     $self->sendcmd(
       $dst, $cmd,
       #map {ref $_ eq 'ARRAY' ? @$_:ref $_ eq 'HASH' ? each : $_)    }@_
-      ( $dst eq 'C' || !length $self->{'sid'} ? () : $self->{'sid'} ), $self->adc_make_string(@_)
+      ( $self->{'broadcast'} ? $self->{'INF'}{'ID'} : $dst eq 'C' || !length $self->{'sid'} ? () : $self->{'sid'} ),
+      $self->adc_make_string(@_)
         #( $dst eq 'D' || !length $self->{'sid'} ? () : $self->{'sid'} ),
     );
   };
@@ -1428,10 +1386,20 @@ sub func {
     s/\D//g;
     return $token + $_ + int time;
   };
+  $self->{'say'} = sub (@) {
+    my $self = shift;
+    @_ = $_[2] if $_[0] eq 'MSG';
+    #$self->log("SAY charset_console=$self->{charset_console} charset_fs=$self->{charset_fs}==== @_" , Dumper \@_);
+    #local $_ = Encode::encode $self->{charset_console} , join ' ', @_;
+    local $_ = Encode::encode $self->{charset_console}, Encode::decode $self->{charset_internal}, join ' ', @_;
+    #}
+    #$self->log("SAY after === $_", Dumper $_);
+    print $_, "\n";
+  };
   local %_ = (
     'search' => sub {
       my $self = shift if ref $_[0];
-      $self->log( 'search', @_ );
+      #$self->log( 'search', @_ );
       return $self->cmd( 'search_tth', @_ ) if length $_[0] == 39 and $_[0] =~ /^[0-9A-Z]+$/;
       return $self->cmd( 'search_string', @_ ) if length $_[0];
     },
