@@ -1,18 +1,19 @@
-#$Id: adc.pm 629 2010-02-12 00:59:11Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect/adc.pm $
+#$Id: adc.pm 686 2010-12-16 00:02:50Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect/adc.pm $
 package    #hide from cpan
   Net::DirectConnect::adc;
 use strict;
 use Time::HiRes qw(time sleep);
+use Socket;
 use Data::Dumper;    #dev only
 $Data::Dumper::Sortkeys = $Data::Dumper::Useqq = $Data::Dumper::Indent = 1;
 #eval "use MIME::Base32 qw( RFC ); 1;"        or print join ' ', ( 'err', 'cant use', $@ );
-use MIME::Base32 qw( RFC );
+#use MIME::Base32 qw( RFC );
 use Net::DirectConnect;
 #use Net::DirectConnect::clicli;
 use Net::DirectConnect::http;
 #use Net::DirectConnect::httpcli;
 no warnings qw(uninitialized);
-our $VERSION = ( split( ' ', '$Revision: 629 $' ) )[1];
+our $VERSION = ( split( ' ', '$Revision: 686 $' ) )[1];
 use base 'Net::DirectConnect';
 our %codesSTA = (
   '00' => 'Generic, show description',
@@ -50,8 +51,9 @@ qq{Direct connection failed, flag "TO" the token, flag "PR" the protocol string.
   '54' => 'No hash support overlap in SUP between clients.',
 );
 #eval "use Net::DirectConnect::TigerHash; 1;" or print join ' ', ( 'err', 'cant use', $@ );
-eval q{use Net::DirectConnect::TigerHash;};
+#eval q{use Net::DirectConnect::TigerHash;};
 
+=no
 sub base32 ($) {
   #eval {
   MIME::Base32::encode( $_[0] );
@@ -68,6 +70,8 @@ sub tiger ($) {
     #mhash(Mhash::MHASH_TIGER, $_);
 }
 sub hash ($) { base32( tiger( $_[0] ) ); }
+=cut
+
 #sub init {  my $self = shift;
 
 =cu
@@ -84,7 +88,6 @@ return $self;
 
 }
 =cut
-
 sub init {
   my $self = shift;
   #shift if $_[0] eq __PACKAGE__;
@@ -104,13 +107,11 @@ sub init {
     #ADC
     'connect_protocol' => 'ADC/0.10', 'message_type' => 'H',
     #@_,
-    'incomingclass'  => __PACKAGE__,                               #'Net::DirectConnect::adc',
-    no_print         => { 'INF' => 1, 'QUI' => 1, 'SCH' => 1, },
-    charset_protocol => 'utf8',
+    'incomingclass' => __PACKAGE__,                        #'Net::DirectConnect::adc',
+    no_print => { 'INF' => 1, 'QUI' => 1, 'SCH' => 1, },
   );
-#  $self->{$_} ||= $_{$_} for keys %_;
-  !exists $self->{$_} ?  $self->{$_} ||= $_{$_} : ()  for keys %_;
-
+  #$self->{$_} ||= $_{$_} for keys %_;
+  !exists $self->{$_} ? $self->{$_} ||= $_{$_} : () for keys %_;
   #print 'adc init now=',Dumper $self;
   $self->{'periodic'}{ __FILE__ . __LINE__ } = sub { $self->cmd( 'search_buffer', ) if $self->{'socket'}; };
   #$self->log( $self, 'inited', "MT:$self->{'message_type'}", ' with', Dumper \@_ );
@@ -127,6 +128,62 @@ sub init {
   }
   $self->{$_} ||= $self->{'parent'}{$_} ||= {} for qw(peers peers_sid peers_cid want share_full share_tth);
   $self->{$_} ||= $self->{'parent'}{$_} for qw(ID PID CID INF SUPAD myport);
+  $self->{message_type} = 'B' if $self->{'broadcast'};
+  if ( Net::DirectConnect::use_try( 'MIME::Base32', 'RFC' ) ) {
+    $self->{base_encode} ||= sub {
+      shift if ref $_[0];
+      MIME::Base32::encode_rfc3548(@_);
+    };
+    $self->{base_decode} ||= sub {
+      shift if ref $_[0];
+      MIME::Base32::decode_rfc3548(@_);
+    };
+  }
+  else {
+    $self->log( 'err', 'cant use MIME::Base32' );
+  }
+  if ( Net::DirectConnect::use_try('Net::DirectConnect::TigerHash') ) {
+    $self->{hash} ||= sub { shift if ref $_[0]; Net::DirectConnect::TigerHash::tthbin( $_[0] ); };
+    $self->{base_encode} ||= sub {
+      shift if ref $_[0];
+      Net::DirectConnect::TigerHash::toBase32( $_[0] );
+    };
+    $self->{base_decode} ||= sub {
+      shift if ref $_[0];
+      Net::DirectConnect::TigerHash::fromBase32( $_[0] );
+    };
+  } else {
+    $self->log( 'err', 'cant use Net::DirectConnect::TigerHash' );
+  }
+  $self->{hash_base} ||= sub { shift if ref $_[0]; $self->base_encode( $self->hash( $_[0] ) ) };
+  #sub hash ($) { base32( tiger( $_[0] ) ); }
+  $self->{INF_generate} ||= sub {
+    my $self = shift if ref $_[0];
+    $self->{'INF'}{'NI'} ||= $self->{'Nick'} || 'perlAdcDev';
+    $self->{'PID'} ||= MIME::Base32::decode $self->{'INF'}{'PD'} if $self->{'INF'}{'PD'};
+    $self->{'CID'} ||= MIME::Base32::decode $self->{'INF'}{'ID'} if $self->{'INF'}{'ID'};
+    $self->{'ID'}  ||= 'perl' . $self->{'myip'} . $self->{'INF'}{'NI'};
+    $self->{'PID'} ||= $self->hash( $self->{'ID'} );
+    $self->{'CID'} ||= $self->hash( $self->{'PID'} );
+    $self->{'INF'}{'PD'} ||= $self->base_encode( $self->{'PID'} );
+    $self->{'INF'}{'ID'} ||= $self->base_encode( $self->{'CID'} );
+#$self->log( 'id gen',"iID=$self->{'INF'}{'ID'} iPD=$self->{'INF'}{'PD'} PID=$self->{'PID'} CID=$self->{'CID'} ID=$self->{'ID'}" );
+    $self->{'INF'}{'SL'} ||= $self->{'S'}         || '2';
+    $self->{'INF'}{'SS'} ||= $self->{'sharesize'} || 20025693588;
+    $self->{'INF'}{'SF'} ||= 30999;
+    $self->{'INF'}{'HN'} ||= $self->{'H'}         || 1;
+    $self->{'INF'}{'HR'} ||= $self->{'R'}         || 0;
+    $self->{'INF'}{'HO'} ||= $self->{'O'}         || 0;
+    $self->{'INF'}{'VE'} ||= $self->{'client'} . $self->{'V'}
+      || 'perl'
+      . $Net::DirectConnect::VERSION . '_'
+      . $VERSION;    #. '_' . ( split( ' ', '$Revision: 686 $' ) )[1];    #'++\s0.706';
+    $self->{'INF'}{'US'} ||= 10000;
+    $self->{'INF'}{'U4'} ||= $self->{'myport_udp'};
+    $self->{'INF'}{'I4'} ||= $self->{'myip'};
+    $self->{'INF'}{'SU'} ||= 'ADC0,TCP4,UDP4';
+    return $self->{'INF'};
+  };
   $self->{'parse'} ||= {
 #
 #=================
@@ -143,7 +200,7 @@ sub init {
       if ( $dst eq 'H' ) {
         $self->cmd( 'I', 'SUP' );
         #$peerid ||= join '', map {} 1..4
-        $peerid ||= base32(
+        $peerid ||= $self->base_encode(
           pack 'S', $self->{'number'}
             #+ int rand 100
         );
@@ -153,7 +210,7 @@ sub init {
         $self->log( 'adcdev', $dst, 'SUP:', @_, "SID:n=$self->{'number'}; $peerid=$self->{'peerid'}" );
         $self->cmd( 'I', 'SID', $peerid );
         $self->cmd( 'I', 'INF', );    #$self->{'peers'}{$_}{'INF'}
-        #for keys %{$self->{'peers'}};
+                                      #for keys %{$self->{'peers'}};
         $self->{'status'} = 'connected';
       }
       $peerid ||= '';
@@ -171,6 +228,7 @@ sub init {
       }
 =cut      
 
+      $self->cmd( 'D', 'INF', ) if $self->{'broadcast'};
       return $self->{'peers'}{$peerid}{'SUP'};
     },
     'SID' => sub {
@@ -202,14 +260,20 @@ sub init {
       my $sendbinf;
       if ( $self->{parent}{hub} and $dst eq 'B' ) {
         if ( !keys %{ $self->{'peers'}{$peerid}{'INF'} } ) {    #join
-          #++$sendbinf;
-          #$self->log( 'adcdev', 'FIRSTINF:', $peerid, Dumper $params, $self->{'peers'} );
+              #++$sendbinf;
+              #$self->log( 'adcdev', 'FIRSTINF:', $peerid, Dumper $params, $self->{'peers'} );
           $self->cmd( 'B', 'INF', $_, $self->{'peers_sid'}{$_}{'INF'} ) for keys %{ $self->{'peers_sid'} };
         }
       }
       #$dst eq 'I' ?
       $self->log( 'adcdev', "ip change from [$params->{I4}] to [$self->{hostip}] " ), $params->{I4} = $self->{hostip}
         if $dst eq 'B' and $self->{parent}{hub} and $params->{I4} and $params->{I4} ne $self->{hostip};   #!$self->{parent}{hub}
+      if ( $dst eq 'B' and $self->{broadcast} ) {
+        $self->log( 'adcdev', "ip change from [$params->{I4}] to [$self->{recv_hostip}:$self->{recv_port}] " );
+        #$params->{U4} = $self->{recv_port};
+        $params->{U4} = $self->{port};
+        $params->{I4} = $self->{recv_hostip};
+      }
       $self->{'peers'}{$peerid}{'INF'}{$_} = $params->{$_} for keys %$params;
       $self->{'peers'}{$peerid}{'object'} = $self;
       $self->{'peers'}{ $params->{ID} }                              ||= $self->{'peers'}{$peerid};
@@ -220,9 +284,9 @@ sub init {
       #$self->log('adcdev', 'INF7', $peerid, @_);
       if ( $dst eq 'I' ) {
         $self->cmd( 'B', 'INF' );
-        $self->{'status'} = 'connected';                                                                  #clihub
+        $self->{'status'} = 'connected';    #clihub
       } elsif ( $dst eq 'C' ) {
-        $self->{'status'} = 'connected';                                                                  #clicli
+        $self->{'status'} = 'connected';    #clicli
         $self->cmd( $dst, 'INF' );
         if   ( $params->{TO} ) { }
         else                   { }
@@ -281,8 +345,8 @@ sub init {
       {
         my $foundedshow = ( $founded =~ m{^/} ? () : '/' ) . (
           #$self->{chrarset_fs}          ?
-#          $self->{charset_fs} ne $self->{charset_protocol} ?
-            Encode::encode $self->{charset_protocol}, Encode::decode $self->{charset_fs}, $founded 
+          #$self->{charset_fs} ne $self->{charset_protocol} ?
+          Encode::encode $self->{charset_protocol}, Encode::decode $self->{charset_fs}, $founded
             #: $founded
         );
         $self->log( 'adcdev', 'SCH', ( $dst, $peerid, 'F=>', @feature ),
@@ -329,6 +393,9 @@ sub init {
       if ( $dst eq 'D' and $self->{'parent'}{'hub'} and ref $self->{'peers'}{$toid}{'object'} ) {
         $self->{'peers'}{$toid}{'object'}->cmd( 'D', 'RES', $peerid, $toid, @_ );
       }
+      if ( exists $self->{'want_download'}{ $params->{'TR'} } ) {
+        $self->{'want_download'}{ $params->{'TR'} }{$peerid} = $params;    #maybe not all
+      }
       $params;
     },
     'MSG' => sub {
@@ -362,7 +429,6 @@ sub init {
         'auto_connect' => 1,
       );
 =cut
-
     },
     'CTM' => sub {
       my $self = shift if ref $_[0];
@@ -417,7 +483,6 @@ sub init {
         $self->log( 'dcerr', 'SND', "unknown type", @_ );
       }
 =cut
-
     },
   };
 
@@ -431,28 +496,29 @@ sub init {
 
 
 =cut  
-
   $self->{'cmd'} = {
     #move to main
     'search_send' => sub {
       my $self = shift if ref $_[0];
       $self->cmd_adc( 'B', 'SCH', @{ $_[0] || $self->{'search_last'} } );
+#$self->send_udp(inet_ntoa(INADDR_BROADCAST), $self->{'dev_broadcast'}, $self->adc_make_string( 'BSCH', @{ $_[0] || $self->{'search_last'} })) if $self->{'dev_broadcast'};
     },
     'search_tth' => sub {
       my $self = shift if ref $_[0];
       $self->{'search_last_string'} = undef;
       $self->log( 'search_tth', @_ );
-      if ( $self->{'adc'} ) { $self->search_buffer( { TO => $self->make_token(), TR => $_[0], } ); }    #toauto
+      local $_ = shift;
+      if ( $self->{'adc'} ) { $self->search_buffer( { TO => $self->make_token(), TR => $_, @_ } ); }    #toauto
       else {
         #$self->cmd( 'search_buffer', 'F', 'T', '0', '9', 'TTH:' . $_[0] );
       }
     },
     'search_string' => sub {
       my $self = shift if ref $_[0];
-      my $string = $_[0];
+      my $string = shift;
       if ( $self->{'adc'} ) {
         #$self->cmd( 'search_buffer', { TO => 'auto', map AN => $_, split /\s+/, $string } );
-        $self->cmd( 'search_buffer', ( map { 'AN' . $_ } split /\s+/, $string ), { TO => $self->make_token(), } );    #TOauto
+        $self->cmd( 'search_buffer', ( map { 'AN' . $_ } split /\s+/, $string ), { TO => $self->make_token(), @_ } );    #TOauto
       } else {
         #$self->{'search_last_string'} = $string;
         #$string =~ tr/ /$/;
@@ -473,10 +539,12 @@ sub init {
     #ADC dev
     #
     'connect_aft' => sub {
-      print "RUNADC![$self->{'protocol'}:$self->{'adc'}]";
+      #print "RUNADC![$self->{'protocol'}:$self->{'adc'}]";
       my $self = shift if ref $_[0];
       #$self->log($self, 'connect_aft inited',"MT:$self->{'message_type'}", ' ');
-      $self->cmd( $self->{'message_type'}, 'SUP' ) if $self->{'adc'};
+      if ( $self->{'broadcast'} ) { $self->cmd( $self->{'message_type'}, 'INF' ); }
+      #else
+      { $self->cmd( $self->{'message_type'}, 'SUP' ); }
     },
     'cmd_all' => sub {
       my $self = shift if ref $_[0];
@@ -519,58 +587,14 @@ sub init {
           return;
         }
       } else {
-        $self->{'INF'}{'NI'} ||= $self->{'Nick'} || 'perlAdcDev';
-        #eval "use MIME::Base32 qw( RFC );  use Digest::Tiger;" or $self->log( 'err', 'cant use', $@ );
-
-=z
-      eval "use MIME::Base32 qw( RFC ); 1;"        or $self->log( 'err', 'cant use', $@ );
-      eval "use Net::DirectConnect::TigerHash; 1;" or $self->log( 'err', 'cant use', $@ );
-      sub base32 ($) { MIME::Base32::encode( $_[0] ); }
-      sub hash ($)   { base32( tiger( $_[0] ) ); }
-
-      sub tiger ($) {
-        local ($_) = @_;
-        #use Mhash qw( mhash mhash_hex MHASH_TIGER);
-        #eval "use MIME::Base32 qw( RFC ); use Digest::Tiger;" or $self->log('err', 'cant use', $@);
-        #$_.=("\x00"x(1024 - length $_));        print ( 'hlen', length $_);
-        #Digest::Tiger::hash($_);
-      eval{  Net::DirectConnect::TigerHash::tthbin($_);}	
-        #mhash(Mhash::MHASH_TIGER, $_);
-      }
-=cut
-
-        #$self->log('tiger of NULL is', hash(''));#''=      LWPNACQDBZRYXW3VHJVCJ64QBZNGHOHHHZWCLNQ
-        #
-        $self->{'PID'} ||= MIME::Base32::decode $self->{'INF'}{'PD'} if $self->{'INF'}{'PD'};
-        $self->{'CID'} ||= MIME::Base32::decode $self->{'INF'}{'ID'} if $self->{'INF'}{'ID'};
-        $self->{'ID'}  ||= 'perl' . $self->{'myip'} . $self->{'INF'}{'NI'};
-        $self->{'PID'} ||= tiger $self->{'ID'};
-        $self->{'CID'} ||= tiger $self->{'PID'};
-        $self->{'INF'}{'PD'} ||= base32 $self->{'PID'};
-        $self->{'INF'}{'ID'} ||= base32 $self->{'CID'};
-        $self->log( 'id gen',
-          "iID=$self->{'INF'}{'ID'} iPD=$self->{'INF'}{'PD'} PID=$self->{'PID'} CID=$self->{'CID'} ID=$self->{'ID'}" );
-        $self->{'INF'}{'SL'} ||= $self->{'S'}         || '2';
-        $self->{'INF'}{'SS'} ||= $self->{'sharesize'} || 20025693588;
-        $self->{'INF'}{'SF'} ||= 30999;
-        $self->{'INF'}{'HN'} ||= $self->{'H'}         || 1;
-        $self->{'INF'}{'HR'} ||= $self->{'R'}         || 0;
-        $self->{'INF'}{'HO'} ||= $self->{'O'}         || 0;
-        $self->{'INF'}{'VE'} ||= $self->{'client'} . $self->{'V'}
-          || 'perl'
-          . $Net::DirectConnect::VERSION . '_'
-          . $VERSION;    #. '_' . ( split( ' ', '$Revision: 629 $' ) )[1];    #'++\s0.706';
-        $self->{'INF'}{'US'} ||= 10000;
-        $self->{'INF'}{'U4'} ||= $self->{'myport_udp'};
-        $self->{'INF'}{'I4'} ||= $self->{'myip'};
-        $self->{'INF'}{'SU'} ||= 'ADC0,TCP4,UDP4';
+        $self->INF_generate();
      #$self->{''} ||= $self->{''} || '';
      #$self->sendcmd( $dst, 'INF', $self->{'sid'}, map { $_ . $self->{$_} } grep { length $self->{$_} } @{ $self->{'BINFS'} } );
       }
-      $self->cmd_adc     #sendcmd
+      $self->cmd_adc        #sendcmd
         (
-        $dst, 'INF',     #$self->{'sid'},
-        map { $_ . $self->{'INF'}{$_} } $dst eq 'C' ? qw(ID TO) : sort keys %{ $self->{'INF'} }
+        $dst, 'INF',        #$self->{'sid'},
+        map { $_ . $self->{'INF'}{$_} } $dst eq 'C' ? qw(ID TO) : @_ ? @_ : sort keys %{ $self->{'INF'} }
         );
       #grep { length $self->{$_} } @{ $self->{'BINFS'} } );
       #$self->cmd_adc( $dst, 'INF', $self->{'sid'}, map { $_ . $self->{$_} } grep { $self->{$_} } @{ $self->{'BINFS'} } );
@@ -583,7 +607,8 @@ sub init {
       #$self->sendcmd( $dst, 'CTM', $self->{'connect_protocol'},@_);
       local @_ = @_;
       if ( !@_ ) {
-        @_ = ( 'file', $self->{'filename'}, '0', '-1' ) if $self->{'filename'};
+        @_ = ( 'file', $self->{'filename'}, $self->{'file_recv_from'} || '0', $self->{'file_recv_to'} || '-1' )
+          if $self->{'filename'};
         $self->log( 'err', "Nothing to get" ), return unless @_;
       }
       $self->cmd_adc( $dst, 'GET', @_ );
@@ -620,7 +645,6 @@ sub init {
       $self->cmd_adc( $dst, 'SND', @_ );
     },
 =cut    
-
   #$self->log( 'dev', "0making listeners [$self->{'M'}]" );
   unless ( $self->{'no_listen'} ) {
     if ( ( $self->{'M'} eq 'A' or !$self->{'M'} ) and !$self->{'auto_listen'} and !$self->{'incoming'} ) {
@@ -654,7 +678,7 @@ sub init {
         'NONONOparse' => {
           'SR'  => $self->{'parse'}{'SR'},
           'PSR' => sub {                     #U
-            #$self->log( 'dev', "UPSR", @_ );
+                                             #$self->log( 'dev', "UPSR", @_ );
           },
 #2008/12/14-13:30:50 [3] rcv: welcome UPSR FQ2DNFEXG72IK6IXALNSMBAGJ5JAYOQXJGCUZ4A NIsss2911 HI81.9.63.68:4111 U40 TRZ34KN23JX2BQC2USOTJLGZNEWGDFB327RRU3VUQ PC4 PI0,64,92,94,100,128,132,135 RI64,65,66,67,68,68,69,70,71,72
 #UPSR CDARCZ6URO4RAZKK6NDFTVYUQNLMFHS6YAR3RKQ NIAspid HI81.9.63.68:411 U40 TRQ6SHQECTUXWJG5ZHG3L322N5B2IV7YN2FG4YXFI PC2 PI15,17,20,128 RI128,129,130,131
@@ -668,6 +692,19 @@ sub init {
       $self->log( 'err', "cant listen udp (search repiles)" ) unless $self->{'myport_udp'};
     }
     #DEV=z
+
+=no
+    if ( $self->{'dev_broadcast'} ) {
+$self->log( 'info', 'listening broadcast ', $self->{'dev_broadcast'} || $self->{'port'});
+      $self->{'clients'}{'listener_udp_broadcast'} = $self->{'incomingclass'}->new(
+        #%$self, $self->clear(),
+        'parent' => $self, 'Proto' => 'udp', 'auto_listen' => 1,
+      'sockopts' => {%{$self->{'sockopts'}||{}}, 'Broadcast'=>1},
+      myport => $self->{'dev_broadcast'} || $self->{'port'},
+      );
+      $self->log( 'err', "cant listen broadcast (hubless)" ) unless $self->{'clients'}{'listener_udp_broadcast'}{'myport'};
+    }
+=cut
     if ( $self->{'dev_http'} ) {
       $self->log( 'dev', "making listeners: http" );
       #$self->{'clients'}{'listener_http'} = Net::DirectConnect::http->new(
@@ -702,8 +739,10 @@ sub init {
     $self->log(
       'dev',  'disconnect int',           #psmisc::caller_trace(30)
       'hub=', $self->{'parent'}{'hub'},
-      ) if $self
+      )
+      if $self
         and $self->{'log'};
+    #psmisc::caller_trace 15;
   };
   $self->get_peer_addr() if $self->{'socket'};
   $self->log( 'err', 'cant load TigerHash module' ) unless $INC{'Net/DirectConnect/TigerHash.pm'};
