@@ -1,4 +1,4 @@
-#$Id: clihub.pm 627 2010-02-10 23:47:13Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect/clihub.pm $
+#$Id: clihub.pm 686 2010-12-16 00:02:50Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect/clihub.pm $
 package    #hide from cpan
   Net::DirectConnect::clihub;
 use strict;
@@ -9,7 +9,7 @@ use Net::DirectConnect;
 use Net::DirectConnect::clicli;
 #use Net::DirectConnect::http;
 no warnings qw(uninitialized);
-our $VERSION = ( split( ' ', '$Revision: 627 $' ) )[1];
+our $VERSION = ( split( ' ', '$Revision: 686 $' ) )[1];
 use base 'Net::DirectConnect';
 
 sub name_to_ip($) {
@@ -58,10 +58,9 @@ sub init {
     #@_,
     'incomingclass' => 'Net::DirectConnect::clicli',
     #'periodic'      =>
-    'disconnect_recursive' => 1,
-    charset_protocol       => 'cp1251',    #'utf8'
+    'disconnect_recursive' => 1, charset_protocol => 'cp1251',    #'utf8'
   );
-  !exists $self->{$_} ?  $self->{$_} ||= $_{$_} : ()  for keys %_;
+  !exists $self->{$_} ? $self->{$_} ||= $_{$_} : () for keys %_;
   $self->{'periodic'}{ __FILE__ . __LINE__ } = sub { $self->cmd( 'search_buffer', ) if $self->{'socket'}; };
   #$self->log($self, 'inited',"MT:$self->{'message_type'}", ' with', Dumper  \@_);
   #$self->baseinit();
@@ -119,7 +118,9 @@ sub init {
           $try =~ s/[^\Q$1\E]//g;
           $self->log( 'warn', "CHNICK $self->{'Nick'} -> $try" );
           $self->{'Nick'} = $try if length $try;
-        } elsif ( $text =~ /Bad nickname: Wait (\d+)sec before reconnecting/i ) {
+        } elsif ( $text =~ /Bad nickname: Wait (\d+)sec before reconnecting/i
+          or $text =~ /  (\d+)    \./ )
+        {
           sleep $1 + 1;
         } elsif ( $self->{'auto_bug'} and $nick eq 'VerliHub' and $text =~ /^This Hub Is Running Version 0.9.8d/i ) {    #_RC1
           ++$self->{'bug_MyINFO_last'};
@@ -231,7 +232,8 @@ sub init {
     'RevConnectToMe' => sub {
       my $self = shift if ref $_[0];
       my ( $to, $from ) = split /\s+/, $_[0];
-      $self->cmd( 'ConnectToMe', $to ) if $from eq $self->{'Nick'};
+      $self->log( 'dev', "[$from eq $self->{'Nick'}] ($_[0])" );
+      $self->log( 'dev', 'go ctm' ), $self->cmd( 'ConnectToMe', $to ) if $from eq $self->{'Nick'};
     },
     'GetPass' => sub {
       my $self = shift if ref $_[0];
@@ -260,7 +262,8 @@ sub init {
       my $founded = $self->{'share_full'}{ $s{'tth'} } || $self->{'share_full'}{ $s{'string'} };
       my $tth = $self->{'share_tth'}{$founded};
       if (
-        $founded and $tth
+            $founded
+        and $tth
         #$s{'tth'} and $self->{'share_tth'}{ $s{'tth'} }
         )
       {
@@ -422,7 +425,8 @@ sub init {
     },
     'ConnectToMe' => sub {
       my $self = shift if ref $_[0];
-      return if $self->{'M'} eq 'P' and !$self->{'allow_passive_ConnectToMe'};
+      $self->log( 'dcdbg', "cannot ConnectToMe from passive mode" ), return
+        if $self->{'M'} eq 'P' and !$self->{'allow_passive_ConnectToMe'};
       $self->log( 'err', "please define myip" ), return unless $self->{'myip'};
       $self->sendcmd( 'ConnectToMe', $_[0], "$self->{'myip'}:$self->{'myport'}" );
     },
@@ -455,25 +459,39 @@ sub init {
       $self->sendcmd( 'Search', ( $self->{'M'} eq 'P' ? "Hub:$self->{'Nick'}" : "$self->{'myip'}:$self->{'myport_udp'}" ),
         join '?', @_ );
     },
+    'search_nmdc' => sub {
+      my $self = shift if ref $_[0];
+      local @_ = @_;
+      $_[0] =~ tr/ /$/;
+      @_ = ( ( 'F', 'T', '0', undef )[ 0 .. 3 - $#_ ], reverse @_ );
+      $_[3] ||= ( $_[4] =~ s/^(TTH:)?([A-Z0-9]{39})$/TTH:$2/ ? '9' : '1' ) unless defined $_[3];
+      #
+      #$self->cmd( 'search_buffer', 'F', 'T', '0', '1', @_ );
+      $self->cmd( 'search_buffer', @_ );
+    },
     'search_tth' => sub {
       my $self = shift if ref $_[0];
       $self->{'search_last_string'} = undef;
-      if ( $self->{'adc'} ) {
-        #$self->cmd( 'search_buffer', { TO => $self->make_token(), TR => $_[0], } );
-      }    #toauto
-      else { $self->cmd( 'search_buffer', 'F', 'T', '0', '9', 'TTH:' . $_[0] ); }
+      $self->cmd(
+        'search_nmdc',
+        #'F', 'T', '0', '9',
+        #'TTH:' .
+        #$_[0],
+        @_
+      );
     },
     'search_string' => sub {
       my $self = shift if ref $_[0];
-      my $string = $_[0];
-      if ( $self->{'adc'} ) {
-        #$self->cmd( 'search_buffer', { TO => 'auto', map AN => $_, split /\s+/, $string } );
-        #$self->cmd( 'search_buffer', ( map { 'AN' . $_ } split /\s+/, $string ), { TO => $self->make_token(), } );    #TOauto
-      } else {
-        $self->{'search_last_string'} = $string;
-        $string =~ tr/ /$/;
-        $self->cmd( 'search_buffer', 'F', 'T', '0', '1', $string );
-      }
+      #my $string = $_[0];
+      $self->{'search_last_string'} = $_[0];    #$string;
+                                                #$string =~ tr/ /$/;
+      $self->cmd(
+        'search_nmdc',
+        #'F', 'T', '0', '1',
+        #$string,
+        @_
+      );
+      #}
     },
     'search_send' => sub {
       my $self = shift if ref $_[0];
@@ -515,7 +533,7 @@ sub init {
       #'PortList'    => \%{ $self->{'PortList'} },
       #'handler'     => \%{ $self->{'handler'} },
       #'share_tth'      => $self->{'share_tth'},
-      'myport' => $self->{myport},
+      'myport'      => $self->{myport},
       'auto_listen' => 1,
       'parent'      => $self,
     );
@@ -524,8 +542,7 @@ sub init {
     $self->log( 'dev', "making listeners: udp" );
     $self->{'clients'}{'listener_udp'} = $self->{'incomingclass'}->new(
       #%$self, $self->clear(),
-      'parent' => $self, 'Proto' => 'udp',
-      'myport' => $self->{myport_udp},
+      'parent' => $self, 'Proto' => 'udp', 'myport' => $self->{myport_udp},
       #?    'want'     => \%{ $self->{'want'} },
       #?    'NickList' => \%{ $self->{'NickList'} },
       #?    'IpList'   => \%{ $self->{'IpList'} },
@@ -547,9 +564,7 @@ sub init {
           my $self = shift if ref $_[0];
           #my $self =  ref $_[0] ? shift() : $self;
           $self->log( 'dev', "UPSR", 'udp' ) if $self;
-          for ( split /\n+/, $_[0] ) {
-            return $self->parser($_) if /^\$SR/;
-          }
+          for ( split /\n+/, $_[0] ) { return $self->parser($_) if /^\$SR/; }
           #$self->log( 'dev', "UPSR", @_ ) if $self;
         },
 #2008/12/14-13:30:50 [3] rcv: welcome UPSR FQ2DNFEXG72IK6IXALNSMBAGJ5JAYOQXJGCUZ4A NIsss2911 HI81.9.63.68:4111 U40 TRZ34KN23JX2BQC2USOTJLGZNEWGDFB327RRU3VUQ PC4 PI0,64,92,94,100,128,132,135 RI64,65,66,67,68,68,69,70,71,72
